-----------------------------------------------------------------------------------------------
-- Client Lua Script for StuffPerSecond
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- StuffPerSecond Module Definition
-----------------------------------------------------------------------------------------------
local StuffPerSecond = {} 


StuffPerSecond.HealQueue = {}
StuffPerSecond.DamageQueue = {}

StuffPerSecond.TimerTime = 0.100
StuffPerSecond.QueueCounter = 5 / StuffPerSecond.TimerTime

StuffPerSecond.DamageTemplate = "CRB_NormalFramedThick"
StuffPerSecond.HealTemplate = "CRB_HologramFramedThick"

StuffPerSecond.DPSShown = true

 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function StuffPerSecond:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function StuffPerSecond:Init()
    Apollo.RegisterAddon(self, false)
end
 

-----------------------------------------------------------------------------------------------
-- StuffPerSecond OnLoad
-----------------------------------------------------------------------------------------------
function StuffPerSecond:OnLoad()
	self.config = Apollo.GetPackage("Gemini:DB-1.0").tPackage:New(self)

	Apollo.RegisterEventHandler("CombatLogHeal", "OnCombatLogHeal", self)
	Apollo.RegisterEventHandler("CombatLogDamage", "OnCombatLogDamage", self)

    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("StuffPerSecond.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function StuffPerSecond:initLocation()
	if self.config.char.Position ~= nil then
		self.wndMain:SetAnchorOffsets(self.config.char.Position.left, self.config.char.Position.top, self.config.char.Position.right, self.config.char.Position.bottom)
	else 
		self.config.char.Position = {}
		self.config.char.Position.left, self.config.char.Position.top, self.config.char.Position.right, self.config.char.Position.bottom = self.wndMain:GetAnchorOffsets()
	end
end

-----------------------------------------------------------------------------------------------
-- StuffPerSecond OnDocLoaded
-----------------------------------------------------------------------------------------------
function StuffPerSecond:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "StuffPerSecondForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self:initLocation()
		
		self.wndMainPS = self.wndMain:FindChild("MainPS")
		self.wndSecondPS = self.wndMain:FindChild("SecondPS")
		
		Apollo.LoadForm(self.xmlDoc, "ContentWnd", self.wndHeal, self)
		Apollo.LoadForm(self.xmlDoc, "ContentWnd", self.wndDamage, self)

	    self.wndMain:Show(true, true)
	
		Apollo.RegisterSlashCommand("sps", "OnWndSwitch", self)

		self.timer = ApolloTimer.Create(self.TimerTime, true, "OnTimer", self)
	end
end

-- on timer
function StuffPerSecond:OnTimer()
	-- Do your timer-related stuff here.
	local HealCounter = 0
	local TotalHeal = 0
	local DamageCounter = 0
	local TotalDamage = 0
	
	for k, d in pairs(self.DamageQueue) do
		DamageCounter = DamageCounter + d.counter
		TotalDamage = (d.damage * d.counter) + TotalDamage 
		d.counter = d.counter - 1
		if d.counter <= 0 then
			self.DamageQueue[k] = nil
		end
	end

	for k, h in pairs(self.HealQueue) do
		HealCounter = HealCounter + h.counter
		TotalHeal = (h.heal * h.counter) + TotalHeal
		h.counter = h.counter - 1
		if h.counter <= 0 then
			self.HealQueue[k] = nil
		end
	end

	local avgDamage = 0
	if DamageCounter > 0 then
		avgDamage = TotalDamage / DamageCounter
	end
	
	local avgHeal = 0
	if HealCounter > 0 then
		avgHeal = TotalHeal / HealCounter
	end
	
	if avgHeal == avgDamage and avgDamage == 0 then
		self.wndMain:Show(false)
	else
		self.wndMain:Show(true)
	end
	
	if(avgDamage >= avgHeal) then
		self.wndMain:SetWindowSubclass(self.DamageTemplate)
		self:UpdateMainValue(self:rewriteValue(avgDamage) .. " dps")
		if avgHeal == 0 then
			self:UpdateSecondValue("")
		else
			self:UpdateSecondValue(self:rewriteValue(avgHeal) .. " hps")
		end
	else
		self.wndMain:SetWindowSubclass(self.HealTemplate)
		self:UpdateMainValue(self:rewriteValue(avgHeal) .. " hps")
		if avgDamage == 0 then
			self:UpdateSecondValue("")
		else
			self:UpdateSecondValue(self:rewriteValue(avgDamage) .. " dps")
		end
	end
end

function StuffPerSecond:round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function StuffPerSecond:OnCombatLogHeal(tEventArgs)
	local info = {
		target = tEventArgs.unitTarget,
		caster = tEventArgs.unitCaster,
		owner = tEventArgs.unitCasterOwner,
		ability = tEventArgs.splCallingSpell,
		damagetype = GameLib.CodeEnumDamageType.Heal,
		heal = (tEventArgs.nHealAmount - tEventArgs.nOverheal),
		overheal = tEventArgs.nOverheal,
		crit = (tEventArgs.eCombatResult == GameLib.CodeEnumCombatResult.Critical),
	}
	
	if not info.caster or not info.target then return end
	if not info.caster:IsThePlayer() then return end

	self:RegisterHeal(info)
end

function StuffPerSecond:OnCombatLogDamage(tEventArgs)
	local info = {
		target = tEventArgs.unitTarget,
		caster = tEventArgs.unitCaster,
		owner = tEventArgs.unitCasterOwner,
		ability = tEventArgs.splCallingSpell,
		damagetype = tEventArgs.eDamageType,
		damage = (tEventArgs.nDamageAmount - tEventArgs.nOverkill),
		periodic = tEventArgs.bPeriodic,
		overkill = tEventArgs.nOverkill,
		crit = (tEventArgs.eCombatResult == GameLib.CodeEnumCombatResult.Critical),
	}
	
	if info.target and
	(info.target:GetName() == "Target Dummy" or
	info.target:GetName() == "Formidable Target Dummy" or
	info.target:GetName() == "Weak Target Dummy") then
		info.damage = tEventArgs.nRawDamage - tEventArgs.nOverkill
	end
	
	-- Work around an issue where overkill == damage, in this case there is actually a bug in the data given to us
	-- and we didn't have purely overkill. In this case don't subtract the overkill damage.
	if tEventArgs.nDamageAmount == tEventArgs.nOverkill then
    	info.damage = tEventArgs.nDamageAmount
	end
	
	if not info.caster or not info.target then return end
	if not info.caster:IsThePlayer() then return end
	
	self:RegisterDamage(info)
end

function StuffPerSecond:RegisterHeal(HEAL)
	--Print("HEAL: " .. HEAL.heal .. ", OVERHEAL: " .. HEAL.overheal)
	local item = {}
	item.counter = self.QueueCounter
	item.heal = HEAL.heal
	table.insert(self.HealQueue, item)
end

function StuffPerSecond:RegisterDamage(DPS)
	--Print("DAMAGE: " .. DPS.damage)
	local item = {}
	item.counter = self.QueueCounter
	item.damage = DPS.damage
	table.insert(self.DamageQueue, item)
end

function StuffPerSecond:rewriteValue(value)
	if value > 1000 then
		local newValue = value / 1000.0
		newValue = self:round(newValue, 1)
		return newValue .. " k"
	else
		return value
	end
end

function StuffPerSecond:UpdateMainValue(value)
	self.wndMainPS:SetText(value)
end

function StuffPerSecond:UpdateSecondValue(value)
	self.wndSecondPS:SetText(value)
end
---------------------------------------------------------------------------------------------------
-- StuffPerSecondForm Functions
---------------------------------------------------------------------------------------------------

function StuffPerSecond:OnWndSwitch()
	if self.DPSShown then
		self.wndDamage:Show(false)
		self.wndHeal:Show(true)
		self.DPSShown = false
	else
		self.wndHeal:Show(false)
		self.wndDamage:Show(true)
		self.DPSShown = true
	end
end

function StuffPerSecond:OnWndChanged( wndHandler, wndControl )
	self.config.char.Position = {}
	
	self.config.char.Position.left, self.config.char.Position.top, self.config.char.Position.right, self.config.char.Position.bottom =  self.wndMain:GetAnchorOffsets()
end

-----------------------------------------------------------------------------------------------
-- StuffPerSecond Instance
-----------------------------------------------------------------------------------------------
local StuffPerSecondInst = StuffPerSecond:new()
StuffPerSecondInst:Init()
